----------------------------------------------------------------
-- Globals
----------------------------------------------------------------
TitanFactions = {}
TitanFactionsDB = {}

TitanFactions.SessionStartValues = {}

TitanFactions.StandingLabels = {
	[1] = "|cff8b0000"..FACTION_STANDING_LABEL1.."|r",	-- 36000 Hated
	[2] = "|cffff1919"..FACTION_STANDING_LABEL2.."|r",	-- 3000 Hostile
	[3] = "|cffff8C00"..FACTION_STANDING_LABEL3.."|r",	-- 3000 Unfriendly
	[4] = "|cffc0c0c0"..FACTION_STANDING_LABEL4.."|r",	-- 3000 Neutral
	[5] = "|cffffffff"..FACTION_STANDING_LABEL5.."|r",	-- 6000 Friendly
	[6] = "|cff19ff19"..FACTION_STANDING_LABEL6.."|r",	-- 12000 Honored
	[7] = "|cff4169e1"..FACTION_STANDING_LABEL7.."|r",	-- 21000 Revered
	[8] = "|cff9932cc"..FACTION_STANDING_LABEL8.."|r",	-- 1000 Exalted
}

TitanFactions.RuneclothCounters = {
	TITAN_FACTIONS_DARKSPEAR = true,
	TITAN_FACTIONS_ORGRIMMAR = true,
	TITAN_FACTIONS_SILVERMOON = true,
	TITAN_FACTIONS_THUNDERBLUFF = true,
	TITAN_FACTIONS_UNDERCITY = true,
	TITAN_FACTIONS_DARNASSUS = true,
	TITAN_FACTIONS_EXODAR = true,
	TITAN_FACTIONS_GNOMEREGAN = true,
	TITAN_FACTIONS_IRONFORGE = true,
	TITAN_FACTIONS_STORMWIND = true,
}

----------------------------------------------------------------
-- Globals
----------------------------------------------------------------
local race = select(2, UnitRace("player"))
local runeclothRep = race:upper() == "HUMAN" and 82 or 75

local db = TitanFactionsDB
local count = TitanFactions.RuneclothCounters
local label = TitanFactions.StandingLabels
local start = TitanFactions.SessionStartValues

----------------------------------------------------------------
-- Local functions
----------------------------------------------------------------
local function Debug(text)
	-- comment out the following line for release!
	DEFAULT_CHAT_FRAME:AddMessage("Titan Factions: "..text)
end

local function Blue(text)
	return "|cff19ff19"..text.."|r"
end
local function Gold(text)
	return "|cffffcc00"..text.."|r"
end
local function Gray(text)
	return "|cff777777"..text.."|r"
end
local function Green(text)
	return "|cff19ff19"..text.."|r"
end
local function Pink(text)
	return "|cffffaec8"..text.."|r"
end
local function Red(text)
	return "|cffff1919"..text.."|r"
end
local function White(text)
	return "|cffffffff"..text.."|r"
end
local function Yellow(text)
	return "|cffffff00"..text.."|r"
end

----------------------------------------------------------------
-- Tooltip text function
----------------------------------------------------------------
function TitanFactions:GetTooltipText()
	local name, standing, min, max, value, war, header, collapsed, watched
	local line, faction, session, gain, runecloth, percent, numbers

	local lines = {}

	for i = 1, GetNumFactions() do
		name, _, standing, min, max, value, war, _, header, _, _ = GetFactionInfo(i)
		if name == TITAN_FACTIONS_INACTIVE then break end

		if header then
			table.insert(lines, White(name))
		else
			if not (standing == 8 and db.hideExalted) then
				if standing > 3 then
					faction = Green(name)
				elseif standing < 3 then
					faction = Red(name)
				else
					faction = Yellow(name)
				end

				if db.showRunecloth and count[name] then
					runecloth = Pink(" ("..math.ceil(((42000 - value) / runeclothRep ) + 0.5)..")")
				else
					runecloth = ""
				end

				if db.showSession then
					if not start[name] then start[name] = value end
					gain = value - start[name]
					if gain > 0 then
						session = "[+"..gain.."] "
					else
						session = "["..gain.."] "
					end
				else
					session = ""
				end

				if db.showPercent then
					percent = math.floor((value - min)/(max - min) * 100).."% "
				else
					percent = ""
				end

				if db.showNumbers then
					numbers = "("..value - min.."/"..max - min..")"
				else
					numbers = ""
				end

				line = faction..runecloth.."\t"..session..percent..numbers
				table.insert(lines, line)
			end
		end
	end
	return table.concat(lines, "\n")
end

----------------------------------------------------------------
-- Session start function
----------------------------------------------------------------
function TitanFactions:StartSession(where)
	Debug("Starting session in "..where)
	local name, barValue
	for i = 1, GetNumFactions() do
		name, _, _, _, _, barValue, _, _, _, _, _ = GetFactionInfo(i)
		if not isHeader then
			TitanFactions.Session[name] = barValue
			Debug(name.." starting at "..barValue)
		end
	end
	TITAN_FACTIONS_SESSION_STARTED = true
end

----------------------------------------------------------------
-- Non-Titan initialization
----------------------------------------------------------------
local function OnEvent(event, ...)
	local self = TitanFactions
	self:StartSession("OnEvent")

	TitanFactionsDBDefault = {
		["hideExalted"] = true,
		["showMonitor"] = true,
		["showNumbers"] = true,
		["showPercent"] = true,
		["showRunecloth"] = true,
		["showSession"] = true,
		["showToGo"] = true,
		["syncMonitor"] = true,
	}

	for option, setting in TitanFactionsDBDefault do
		if TitanFactionsDB[option] == nil then
			TitanFactionsDB[option] = setting
		end
	end
end

local f = CreateFrame("Frame", "TitanFactionsFrame", UIParent)
f:RegisterEvent("VARIABLES_LOADED")
f:SetScript("OnEvent", OnEvent)